import java.io.IOException;
import java.net.SocketException;

/**
 * Abstrakte Serverklasse. Diese Klasse darf nicht veraendert werden.
 * 
 */
public abstract class AServer {

    /**
     * Konstuktor. Erstellt einen neuen Server und baut eine lauschende
     * Verbindung am Port port auf.
     * 
     * @param port
     * @throws IOException
     */
    public AServer(int port) throws IOException {
        openPort(port);
        Runnable r = new Runnable() {
            public void run() {
                while (isConnected()) {
                    try {
                        acceptConnection();
                    } catch (SocketException se) {
                        if (!se.getMessage().equalsIgnoreCase("Socket closed")) {
                            se.printStackTrace();
                        }
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        };
        new Thread(r).start();
    }

    /**
     * Sendet die Nachricht message an den Client mit Index index.
     * 
     * @param index
     *            Index des Clients
     * @param message
     *            Zu sendende Nachricht
     * @throws IOException
     */
    public abstract void sendMessage(int index, String message)
            throws IOException;

    /**
     * Wartet auf und empfaengt eine Nachricht vom Client mit Index index.
     * 
     * @param index
     *            Index des Clients
     * @return erhaltene Nachricht
     * @throws IOException
     */
    public abstract String receiveMessage(int index) throws IOException;

    /**
     * Liefert die Anzahl der verbundenen Clients.
     * 
     * @return Anzahl der verbundenen Clients
     */
    public abstract int connectionCount();

    /**
     * Wartet darauf, dass sich ein neuer Client verbindet. Die Verbindung kommt
     * nur zu Stande, wenn der Client direkt nach der Verbindung seine ID
     * uebermittelt. Die Verbindung muss festgehalten werden. Wird keine ID
     * uebermittelt, wird die Verbindung abgebrochen. Ist eine ID bereits vorhanden, so wird eine Fehlermeldung ausgegeben und die Verbindung abgebrochen.
     * 
     * @throws IOException
     */
    abstract void acceptConnection() throws IOException;

    /**
     * Oeffnet einen Port auf dem Server, um Clientverbindungen zu zulassen.
     * 
     * @param port
     * @throws IOException
     */
    abstract void openPort(int port) throws IOException;

    /**
     * Beendet alle offenen Verbindungen zu den Clients und schliesst den
     * lauschenden Port.
     */
    abstract void disconnect();

    /**
     * Beendet die Verbindung mit Client index.
     * 
     * @param index
     *            Index des Clients
     */
    abstract void disconnect(int index);

    /**
     * Gibt an, ob der lauschende Port geoeffnet ist.
     * @return true, wenn der lauschende Port offen ist. false sonst.
     */
    abstract boolean isConnected();

    /**
     * Gibt an, ob eine Verbindung zu dem Client mit Index index besteht.
     * @param index Index des Clients 
     * @return true, wenn eine Verbindung besteht. false sonst.
     */
    abstract boolean isConnected(int index);
}
